#include "RunLights412613398.h"
#include "simcode.h"
#include "math.h"

RunLights412613398::RunLights412613398(){
}

RunLights412613398::~RunLights412613398(){
}

void RunLights412613398::Initialize(const volatile BLOCK_CONSTRUCTOR_PARAM_STRUCT *arg){
    m_prerun_count = 0;
}

//#define USE_HAL_API

#ifdef USE_HAL_API
#define BTN0_PIN 20 //CASP internal id of BTN0
#define BTN1_PIN 21 //CASP internal id of BTN1
#define LED4_PIN 16 //CASP internal id of LED4
#define LED5_PIN 17 //CASP internal id of LED5
#define LED6_PIN 18 //CASP internal id of LED6
#define LED7_PIN 19 //CASP internal id of LED7
#endif

void RunLights412613398::PreRun(const volatile GLOBAL_PRERUN_PARAM_STRUCT *arg, _FLOAT32 *in, _FLOAT32 *out){
    if(m_prerun_count++ != 1) return;
    //initialize variables
    m_prev_time = millis();
    m_cur_out_idx = output_size-1;
    out[m_cur_out_idx] = 1;
    //
#ifdef USE_HAL_API
    pinMode(BTN0_PIN, INPUT);
    pinMode(BTN1_PIN, INPUT);
    pinMode(LED4_PIN, OUTPUT);
    pinMode(LED5_PIN, OUTPUT);
    pinMode(LED6_PIN, OUTPUT);
    pinMode(LED7_PIN, OUTPUT);
#endif
}

void RunLights412613398::Run(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, _FLOAT32 *in, _FLOAT32 *out){
#ifdef USE_HAL_API
    if(input_size > 0) in[0] = digitalRead(BTN0_PIN);
    if(input_size > 1) in[1] = digitalRead(BTN1_PIN);
#endif
    //first update rate based on input data
    int rate = 1;
    for(int i=0; i<input_size; ++i)
        rate += (in[i]*(i+1));
    //update output w.r.t the rate
    _UINT64 cur_time = millis();
    _INT64 diff = cur_time - m_prev_time;
    if(diff > (1000/(base_rate*rate)))
    {
        out[m_cur_out_idx] = 0;
        if(++m_cur_out_idx == output_size)
            m_cur_out_idx = 0;
        out[m_cur_out_idx] = 1;
        m_prev_time = cur_time;
    }
#ifdef USE_HAL_API
    if(output_size > 0) digitalWrite(LED4_PIN, out[0]);
    if(output_size > 1) digitalWrite(LED5_PIN, out[1]);
    if(output_size > 2) digitalWrite(LED6_PIN, out[2]);
    if(output_size > 3) digitalWrite(LED7_PIN, out[3]);
#endif
}

void RunLights412613398::RunPlotStep(const volatile GLOBAL_RUN_PARAM_STRUCT *arg, _FLOAT32 *in, _FLOAT32 *out){
    Run(arg, in, out);
}

void RunLights412613398::PostRun(const volatile GLOBAL_POSTRUN_PARAM_STRUCT *arg, _FLOAT32 *in, _FLOAT32 *out){
}

